@@ -1,14 +1,27 @@
 /*
-  $Header: /cvs/src/chrony/clientlog.h,v 1.5 1999/09/01 21:31:34 richard Exp $
+  $Header: /cvs/src/chrony/clientlog.h,v 1.9 2003/09/22 21:22:30 richard Exp $
 
   =======================================================================
 
   chronyd/chronyc - Programs for keeping computer clocks accurate.
 
-  Copyright (C) 1997-1999 Richard P. Curnow
-  All rights reserved.
-
-  For conditions of use, refer to the file LICENCE.
+ **********************************************************************
+ * Copyright (C) Richard P. Curnow  1997-2003
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * 
+ **********************************************************************
 
   =======================================================================
 
@@ -25,7 +38,7 @@
 typedef unsigned long CLG_IP_Addr;
 
 /* Enough to hold flags for 256 hosts in a class C */
-typedef unsigned long CLG_Bitmap[8];
+typedef uint32_t CLG_Bitmap[8];
 
 extern void CLG_Initialise(void);
 extern void CLG_Finalise(void);
