@@ -1,14 +1,27 @@
 /*
-  $Header: /cvs/src/chrony/logging.c,v 1.9 1999/04/19 20:27:29 richard Exp $
+  $Header: /cvs/src/chrony/logging.c,v 1.15 2003/09/22 21:22:30 richard Exp $
 
   =======================================================================
 
   chronyd/chronyc - Programs for keeping computer clocks accurate.
 
-  Copyright (C) 1997-1999 Richard P. Curnow
-  All rights reserved.
-
-  For conditions of use, refer to the file LICENCE.
+ **********************************************************************
+ * Copyright (C) Richard P. Curnow  1997-2003
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * 
+ **********************************************************************
 
   =======================================================================
 
@@ -74,11 +87,11 @@ LOG_Line_Function(LOG_Severity severity, LOG_Facility facility, const char *form
   char buf[2048];
   va_list other_args;
   va_start(other_args, format);
-  vsprintf(buf, format, other_args);
+  vsnprintf(buf, sizeof(buf), format, other_args);
   va_end(other_args);
 #ifdef WINNT
   if (logfile) {
-    fprintf(logfile, "%s", buf);
+    fprintf(logfile, "%s\n", buf);
   }
 #else
   if (is_detached) {
@@ -95,7 +108,7 @@ LOG_Line_Function(LOG_Severity severity, LOG_Facility facility, const char *form
         break;
     }
   } else {
-    fprintf(stderr, "%s", buf);
+    fprintf(stderr, "%s\n", buf);
   }
 #endif
   return;
@@ -109,18 +122,18 @@ LOG_Fatal_Function(LOG_Facility facility, const char *format, ...)
   char buf[2048];
   va_list other_args;
   va_start(other_args, format);
-  vsprintf(buf, format, other_args);
+  vsnprintf(buf, sizeof(buf), format, other_args);
   va_end(other_args);
 
 #ifdef WINNT
   if (logfile) {
-    fprintf(logfile, "Fatal error : %s", buf);
+    fprintf(logfile, "Fatal error : %s\n", buf);
   }
 #else
   if (is_detached) {
     syslog(LOG_CRIT, "Fatal error : %s", buf);
   } else {
-    fprintf(stderr, "Fatal error : %s", buf);
+    fprintf(stderr, "Fatal error : %s\n", buf);
   }
 #endif
 
@@ -167,13 +180,23 @@ LOG_GoDaemon(void)
 
   if (pid < 0) {
     LOG(LOGS_ERR, LOGF_Logging, "Could not detach, fork failed : %s", strerror(errno));
+  } else if (pid > 0) {
+    exit(0); /* In the 'grandparent' */
   } else {
-    if (pid > 0) {
-      exit(0); /* In the parent */
+
+    setsid();
+
+    /* Do 2nd fork, as-per recommended practice for launching daemons. */
+    pid = fork();
+
+    if (pid < 0) {
+      LOG(LOGS_ERR, LOGF_Logging, "Could not detach, fork failed : %s", strerror(errno));
+    } else if (pid > 0) {
+      exit(0); /* In the 'parent' */
     } else {
+      /* In the child we want to leave running as the daemon */
 
-      setsid();
-
+      /* Don't keep stdin/out/err from before. */
       for (fd=0; fd<1024; fd++) {
         close(fd);
       }
@@ -183,7 +206,7 @@ LOG_GoDaemon(void)
       openlog("chronyd", LOG_PID, LOG_DAEMON);
 
       LOG(LOGS_INFO, LOGF_Logging, "chronyd version %s starting", PROGRAM_VERSION_STRING);
-  
+
     }
   }
 
