@@ -1,14 +1,27 @@
 /*
-  $Header: /cvs/src/chrony/ntp_core.h,v 1.13 1999/04/19 20:27:29 richard Exp $
+  $Header: /cvs/src/chrony/ntp_core.h,v 1.16 2002/02/28 23:27:12 richard Exp $
 
   =======================================================================
 
   chronyd/chronyc - Programs for keeping computer clocks accurate.
 
-  Copyright (C) 1997-1999 Richard P. Curnow
-  All rights reserved.
-
-  For conditions of use, refer to the file LICENCE.
+ **********************************************************************
+ * Copyright (C) Richard P. Curnow  1997-2002
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of version 2 of the GNU General Public License as
+ * published by the Free Software Foundation.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * 
+ **********************************************************************
 
   =======================================================================
 
@@ -86,4 +99,7 @@ extern int NCR_CheckAccessRestriction(unsigned long ip_addr);
 
 extern void NCR_CycleLogFile(void);
 
+extern void NCR_IncrementActivityCounters(NCR_Instance inst, int *online, int *offline, 
+                                          int *burst_online, int *burst_offline);
+
 #endif /* GOT_NTP_CORE_H */
